const fetch = require('node-fetch');
const errors = require('../../errors');
const { to, buildError, isJsonResponse } = require('../../utils');

const getBody = async (res) => await (isJsonResponse(res) ? res.json() : res.text());

const send = async (uri, opts) => {
    const [err, res] = await to(fetch(uri, opts));
    if (err) {
        throw buildError(errors.Type.APIRequestError, err);
    }
    const [getBodyErr, body] = await to(getBody(res));
    if (getBodyErr) {
        throw buildError(errors.Type.APIRequestError, err, 'Failed to read response body');
    }
    if (!res.ok) {
        throw buildError(errors.Type.APIRequestError, new Error(`${res.status} ${res.statusText}: ${JSON.stringify(body)}`));
    }

    return body;
};

function getContextImpl({ apiHost, apiKey }) {
    return async (type, opts) => {
        switch (type) {
            case 'git': {
                const name = opts.git || opts.context;
                return await send(`${apiHost}/api/contexts/${name}/prepare`, {
                    headers: {
                        'Authorization': apiKey,
                    },
                });
            }
            case 'secret-store': {
                const { name } = opts.context;
                return await send(`${apiHost}/api/contexts/${name}/prepare`, {
                    headers: {
                        'Authorization': apiKey,
                    },
                });
            }
            case 'kubernetes-cluster': {
                const name = opts.selector;
                return await send(`${apiHost}/api/clusters/${name}/credentials`, {
                    headers: {
                        'Authorization': apiKey,
                    },
                });
            }
            default:
                throw new Error(`unknown context type: ${type}`);
        }
    };
}

module.exports = getContextImpl;
