const _ = require('lodash');
const Vault = require('../../../clients/hashicorp-vault');
const errors = require('../../../errors');
const { buildError } = require('../../../utils');

/**
 * resolve a variable from a k8s secret
 * @param {*} ctx - secret-context
 * @param {getContextFn} getContext - a function used by the variable resolver to retrieve contexts
 * @param {string} variable - the reference
 * @returns {Promise<string>} resolvedVariable
 */
async function resolve(ctx, getContext, variable) {
    const client = new Vault(_.get(ctx, 'spec.data.apiUrl'), _.get(ctx, 'spec.data.auth'));
    await client.init();

    const { path, key } = Vault.getReferences(ctx, variable);
    const res = await client.read(path);

    const value = _.get(res, `data.data.${key}`, _.get(res, `data.${key}`));
    if (!value) {
        throw buildError(errors.Type.IntegrationKeyNotFoundError, null, `key: "${key}" could not be found in secret: "${path}"`);
    }

    return value;
}

module.exports = { resolve };
